/* 
 * Copyright (C) 2005 Piotr Pokora <piotr.pokora@infoglob.pl>
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *   
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *   */

#ifndef MIDGARD_OBJECT_H
#define MIDGARD_OBJECT_H

/**
 * \defgroup moc Midgard Object Class 
 *
 * This part of API focuses on reflection possibilities for Midgard Object Class.
 * Introspected MidgardObjectClass can be any class which is registered through
 * MidgardSchema ( user defined class ).
 *
 */

#include <glib.h>
#include <midgard/midgard_object.h>

/*
 * \ingroup moc
 *
 * Returns the name of object's primary property.
 *
 * This is static method.
 * 
 * /param klass MidgardObjectClass pointer
 *  
 * /return primary property name
 * 
 * Returned string is a pointer to object's primary property.
 * By default , 'guid' property is primary property which uniquely identifies
 * object's record and object itself. However in some circumstances ( like 
 * backward compatibility ) id property can be defined as primary property.
 *
 * You should not use this function.  
 *
 */           
extern const gchar *midgard_object_class_get_primary_property(MidgardObjectClass *klass); 

/*
 * \ingroup moc
 *
 * Returns the name of object's parent property.
 *
 * This is static method.
 *  
 * /param klass MidgardObjectClass pointer
 *
 * /return parent property name
 * 
 * Returned string is a pointer to object's parent property.
 * Parent property is a property which holds value of field which points to 
 * parent object's identifier. Parent property always points to object's 
 * identifier which is not the same type as given MidgardObjectClass klass.
 *
 * NULL is returned if parent property is not found.
 */
extern const gchar *midgard_object_class_get_property_parent(MidgardObjectClass *klass);

/*
 * \ingroup moc
 *
 * Returns the name of object's up property.
 *
 * This is static method.
 * 
 * /param klass MidgardObjectClass pointer
 * 
 * /return up property name
 * 
 * Returned string is a pointer to object's up property.
 * Up property is a property which holds value of field which points to 
 * "upper" object's identifier. Up property always points to object's
 * dentifier which is the same type as given MidgardObjectClass klass.
 *
 * NULL is returned if up property is not found.
 *
 */
extern const gchar *midgard_object_class_get_property_up(MidgardObjectClass *klass);

/**
 * \ingruop moc
 *
 * Returns array of childreen tree classes for the given MidgardObjectClass.
 *
 * This is static method.
 *
 * \param klass , MidgardObjectClass pointer
 *
 * Returns newly allocated children tree classes' pointers. 
 * NULL is added as the last array element. 
 * Returned array should be freed if no longer needed without freeing array's elements. 
 * ( Use g_free instead of g_strfreev )
 */
extern MidgardObjectClass **midgard_object_class_list_children(
	MidgardObjectClass *klass);

/**
 * \ingroup moc
 * 
 * Checks whether given MidgardObjectClass is multilingual.
 *
 * This is static method.
 * 
 * \param[in] object MidgardObjectClass
 * 
 * \return TRUE if MidgardObjectClass is mutlilingual, FALSE otherwise.
 */
extern gboolean midgard_object_class_is_multilang(MidgardObjectClass *klass);

/**
 * \ingruop moc
 *
 * Returns Midgard Object identified by guid
 *
 * This is static method.
 *
 * \param mgd, MidgardConnection object
 * \param guid, guid string which identifies object
 * 
 * \return MgdObject or NULL
 *
 * MidgardError set by this function:
 *   - MGD_ERR_NOT_EXISTS if object is not found
 *   - MGD_ERR_OBJECT_DELETED is object is already deleted
 *   - MGD_ERR_OBJECT_PURGED is object is purged
 *   - MGD_ERR_SITEGROUP_VIOLATION if more than one record exists 
 *
 *   MGD_ERR_SITEGROUP_VIOLATION is returned if function is invoked by root
 *   user and additional sitegroup constraint has been not defined.
 *
 */
extern MgdObject *midgard_object_class_get_object_by_guid(
			MidgardConnection *mgd, const gchar *guid);

/** 
 * \ingroup moc
 *
 * Get Midgard Object for the given path.
 *
 * This is static method.
 *
 * \param mgd, MidgardConnection object
 * \param classname, name of the class for which object should be returned
 * \param object_path, a path where object should be found
 *
 * \return MgdObject's /c classname instance or NULL if object is not found
 *
 * Cases to set NULL:
 *  - object is not found by path
 *  - object's class has no name or id member
 *  - object's class has no parent and up property
 *
 * MidgardError set by this method:
 *  - MGD_ERR_INTERNAL 
 */
extern MgdObject *midgard_object_class_get_object_by_path(MidgardConnection *mgd,
                const gchar *classname, const gchar *object_path);

/**
 * \ingroup moc
 *
 * Undelete midgard object identified by given guid
 *
 * \param mgd, MidgardConnection object
 * \param guid, string which identifies object
 *
 * \return TRUE on success, FALSE otherwise
 *
 * MidgardError set by this method:
 *  - MGD_ERR_NOT_EXISTS
 *  - MGD_ERR_OBJECT_PURGED
 *  - MGD_ERR_INTERNAL
 */
extern gboolean midgard_object_class_undelete(MidgardConnection *mgd, const gchar *guid);

#endif /* MIDGARD_OBJECT_H */

